/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, bernhard.haubold@fh-weihenstephan.de
 * File created on Sun Jun 20 13:12:10 2004.
 *****************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <limits.h>
#include "interface.h"
#include "eprintf.h"
#include "stringUtil.h"

Args *getArgs(int argc, char *argv[]){
  Args *args;
  char c;
  int arg;

  args = (Args *)emalloc(sizeof(Args));

  args->i = NULL;
  args->o = NULL;
  args->a = 0;
  args->h = 0;
  args->e = 0;
  args->s = 0;
  args->p = 0;
  args->c = 0;
  args->q = 0;
  args->m = INT_MAX;
  args->M = INT_MIN;
  args->n = NAMELENGTH;
  arg = 1;
  while(arg < argc){
    c = argv[arg][1];
      switch(c){
      case 'i':                           /* input file */
	args->i = argv[++arg];
	break;
      case 'o':                           /* output file */
	args->o = argv[++arg];
	break;
     case 's':                            /* separate input sequences */
	args->s = 1;
	break;
     case 'r':                            /* include reverse strand */
	args->r = 1;
	break;
      case 'c':                            /* include reverse strand */
	args->c = 1;
	break;
      case 'p':                           /* print program information test */
	args->p = 1;
	break;
      case 'q':                           /* quietness level */
	args->q = atoi(argv[++arg]);
	break;
      case 'm':                           /* minimal repeat length */
	args->m = atoi(argv[++arg]);
	break;	
      case 'M':                           /* maximal repeat length */
	args->M = atoi(argv[++arg]);
	break;
      case 'n':                            /* number of characters from header line printed */
	args->n = atoi(argv[++arg]);
	if(args->n < 0)
	  args->n = INT_MAX;
	break;
      case 'a':                           /* only maximal repeats? */
	args->a = 1;
	break;
      case 'h':                           /* print help */
	args->h = 1;
	break;
      default:
	printf("# unknown argument: %c\n",c);
	args->e = 1;
	return args;
      }
      arg++;
  }
  if(args->q < 0 || args->q > 2)
    args->e = 1;
  return args;
}


void printUsage(char *version){
  printf("repeater version %s, copyright (c) 2005 Bernhard Haubold\n", version);
  printf("purpose: find maximal exact repeats\n");
  printf("usage: repeater [options]\n");
  printf("options\n");
  printf("\t[-i <FILE> read input from FILE; default: FILE=stdin]\n");
  printf("\t[-o <FILE> write output to FILE; default: FILE=stdout]\n");  
  printf("\t[-m <NUM> look for repeats of length >= NUM; default: longest]\n");	
  printf("\t[-M <NUM> look for repeats of length <= NUM; default: longest]\n");								   
  printf("\t[-q <0|1|2> quietness; default: 0]\n");
  printf("\t[-n <NUM> print first NUM characters of header line; default: %d; prints everything if NUM<0]\n",NAMELENGTH);		       
  printf("\t[-a maximal repeats only; default: all repeats]\n");	
  printf("\t[-r include reverse strand; default: forward only]\n");  
  printf("\t[-s treat each sequence separately; default: union]\n");  
  printf("\t[-c case sensitive; default: ignore case]\n");				  
  printf("\t[-p print information about program]\n");			     
  printf("\t[-h print this help message]\n");
}

void printSplash(char *version){
  printf("********************************************\n");
  printf("*                Repeater %s              *\n",version);
  printf("*                    by                    *\n");
  printf("*             Bernhard Haubold             *\n");
  printf("*------------------------------------------*\n");
  printf("* CONTACT                                  *\n");
  printf("* bernhard.haubold@fh-weihenstephan.de     *\n");
  printf("* ACKNOWLEDGEMENT                          *\n");
  printf("* repeater incorporates parts of the       *\n");
  printf("* strmat software package by Dan Gusfield  *\n");
  printf("* and others. This is available at         *\n");
  printf("* www.cs.ucdavis.edu/~gusfield/strmat.html *\n");
  printf("********************************************\n");
}
