/***** repeater.c *************************************************
 * Description: Search for exact repeats.
 * Author: Bernhard Haubold, bernhard.haubold@fh-weihenstephan.de
 * File created on Thu Jun  9 14:10:46 2005.
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "stree_ukkonen.h"
#include "eprintf.h"
#include "interface.h"
#include "stree_strmat.h"
#include "repeater.h"
#include "sequenceData.h"
#include "stringUtil.h"

int main(int argc, char *argv[]){
  char *version;
  Sequence *forwardStrand = NULL;
  Sequence *reverseStrand = NULL;
  FILE *fpin, *fpout;
  SUFFIX_TREE tree;
  int numString;
  Args *args;

  version = "0.10";
  setprogname2("repeater");

  args = getArgs(argc, argv);
  /* pass args to suffix tree computations */
  setArgs(args);

  if(args->h == 1){ 
    printUsage(version);
    return 0; 
  }else if(args->e == 1){
    printUsage(version);
    return -1;
  }else if(args->p){
    printSplash(version);
    exit(0);
  }

  tree = stree_new_tree(128, 1, 1, 10);
  tree->totalStringLength = 0;
  if(args->i == NULL){
    fpin = stdin;
  }else{
    fpin = efopen(args->i,"r");
  }if(args->o != NULL){       
    fpout = efopen(args->o,"w");
  }else{
    fpout = stdout;
  }

  tree->fpout = fpout;
  numString = 0;
  while((forwardStrand = getPermanentNextSequence(fpin)) != NULL){
    if(!args->c)
      strtoupper(forwardStrand->seq);
    stree_ukkonen_add_string(tree, forwardStrand->seq, forwardStrand->seq, strlen(forwardStrand->seq), numString++, forwardStrand->id);
    if(args->r){
      reverseStrand = revcomp(forwardStrand);
      stree_ukkonen_add_string(tree, reverseStrand->seq, reverseStrand->seq, strlen(reverseStrand->seq), numString++, reverseStrand->id);
    }
    if(args->s){
      int_stree_set_idents(tree);
      fprintf(tree->fpout,"%s|",forwardStrand->id);
      findRepeats(tree);
      stree_delete_tree(tree);
      tree = stree_new_tree(128, 1, 1, 10);
      tree->totalStringLength = 0.0;
      tree->fpout = fpout;
    }
  }
  if(!args->s){
    int_stree_set_idents(tree);
    findRepeats(tree);
  }

  return 0;
}
